----------------------------------------------------------------------
-- TITLE : DVI test (for DE0-nano)
--
--     VERFASSER : S.OSAFUNE (J-7SYSTEM Works)
--     DATUM     : 2012/09/04 -> 2012/09/04 (HERSTELLUNG)
--               : 2013/01/31 (FESTSTELLUNG)
----------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity dvi_test_de0nano is
	generic (
		RESOLUTION		: string := "VGA"		-- 25.175MHz,640x480,60Hz,4:3
--		RESOLUTION		: string := "SVGA"		-- 40.000MHz,800x600,60Hz,4:3
--		RESOLUTION		: string := "XGA"		-- 65.000MHz,1024x768,60Hz,4:3
--		RESOLUTION		: string := "480p"		-- 27.000MHz,720x480,60Hz/59.94Hz,4:3/16:9
--		RESOLUTION		: string := "720p"		-- 74.250MHz,1280x720,60Hz/59.94Hz,16:9
--		RESOLUTION		: string := "1080p/30"	-- 74.250MHz,1920x1080,30Hz/29.97Hz,16:9
	);
	port (
		reset_n			: in  std_logic;
		clk_50mhz		: in  std_logic;		-- input clock 50MHz

		test_hsync_n	: out std_logic;		-- GPIO1_D31
		test_vsync_n	: out std_logic;		-- GPIO1_D30
		test_enable		: out std_logic;		-- GPIO1_D29
		test_locked		: out std_logic;		-- LED[0]

		dvi_data0_p		: out std_logic;		-- GPIO1_D16
		dvi_data0_n		: out std_logic;		-- GPIO1_D17
		dvi_data1_p		: out std_logic;		-- GPIO1_D20
		dvi_data1_n		: out std_logic;		-- GPIO1_D21
		dvi_data2_p		: out std_logic;		-- GPIO1_D13
		dvi_data2_n		: out std_logic;		-- GPIO1_D12
		dvi_clock_p		: out std_logic;		-- GPIO1_D9
		dvi_clock_n		: out std_logic			-- GPIO1_D8
	);
end dvi_test_de0nano;

architecture RTL of dvi_test_de0nano is
	signal reset_n_sig		: std_logic;
	signal videoclk_sig		: std_logic;
	signal locked_sig		: std_logic;

	signal hsync_sig		: std_logic;
	signal vsync_sig		: std_logic;
	signal hblank_sig		: std_logic;
	signal vblank_sig		: std_logic;
	signal de_sig			: std_logic;
	signal hcount_sig		: std_logic_vector(10 downto 0);
	signal vcount_sig		: std_logic_vector(9 downto 0);

	signal pix_r_sig		: std_logic_vector(7 downto 0);
	signal pix_g_sig		: std_logic_vector(7 downto 0);
	signal pix_b_sig		: std_logic_vector(7 downto 0);


	component pll_video_vga
	PORT (
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;			-- 25.175MHz
		locked		: OUT STD_LOGIC 
	);
	end component;
	component pll_video_svga
	PORT (
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;			-- 40.0MHz
		locked		: OUT STD_LOGIC 
	);
	end component;
	component pll_video_xga
	PORT (
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;			-- 65.0MHz
		locked		: OUT STD_LOGIC 
	);
	end component;
	component pll_video_sd
	PORT (
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;			-- 27.0MHz
		locked		: OUT STD_LOGIC 
	);
	end component;
	component pll_video_hd
	PORT (
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;			-- 74.25MHz
		locked		: OUT STD_LOGIC 
	);
	end component;


	component vga_syncgen
	generic (
		H_TOTAL		: integer;
		H_SYNC		: integer;
		H_BACKP		: integer;
		H_ACTIVE	: integer;
		V_TOTAL		: integer;
		V_SYNC		: integer;
		V_BACKP		: integer;
		V_ACTIVE	: integer
	);
	port (
		reset		: in  std_logic;		-- active high
		video_clk	: in  std_logic;		-- typ 25.175MHz

		scan_ena	: in  std_logic;		-- framebuff scan enable
		dither_ena	: in  std_logic;		-- dither enable

		test_hcount	: out std_logic_vector(10 downto 0);	-- H:0`2047
		test_vcount	: out std_logic_vector(9 downto 0);		-- V:0`1023

		framestart	: out std_logic;
		linestart	: out std_logic;
		dither		: out std_logic;		-- RGB444 dither signal
		pixelena	: out std_logic;		-- pixel readout active

		hsync		: out std_logic;
		vsync		: out std_logic;
		hblank		: out std_logic;
		vblank		: out std_logic
	);
	end component;

	component dvi_tx_pdiff
	generic(
		RESET_LEVEL		: std_logic := '1';	-- Positive logic reset
		RESOLUTION		: string := "VGA"	-- 25.175MHz
	);
	port(
		reset		: in  std_logic;
		clk			: in  std_logic;		-- Rise edge drive clock

		test_data0	: out std_logic_vector(9 downto 0);
		test_data1	: out std_logic_vector(9 downto 0);
		test_data2	: out std_logic_vector(9 downto 0);
		test_locked	: out std_logic;

		dvi_de		: in  std_logic;
		dvi_blu		: in  std_logic_vector(7 downto 0);
		dvi_grn		: in  std_logic_vector(7 downto 0);
		dvi_red		: in  std_logic_vector(7 downto 0);
		dvi_hsync	: in  std_logic;
		dvi_vsync	: in  std_logic;
		dvi_ctl		: in  std_logic_vector(3 downto 0) :="0000";

		data0_p		: out std_logic;
		data0_n		: out std_logic;
		data1_p		: out std_logic;
		data1_n		: out std_logic;
		data2_p		: out std_logic;
		data2_n		: out std_logic;
		clock_p		: out std_logic;
		clock_n		: out std_logic
	);
	end component;

begin

	test_hsync_n <= not hsync_sig;
	test_vsync_n <= not vsync_sig;
	test_enable  <= de_sig;
	test_locked  <= locked_sig;


VGA : if (RESOLUTION = "VGA") generate
	pll_inst : pll_video_vga PORT MAP (
		areset	 => not reset_n,
		inclk0	 => clk_50mhz,
		c0	 	 => videoclk_sig,	-- 25.175MHz
		locked	 => reset_n_sig
	);

	vgen_inst : vga_syncgen
	generic map (
		H_TOTAL		=> 800,		-- VGA
		H_SYNC		=> 96,
		H_BACKP		=> 48,
		H_ACTIVE	=> 640,
		V_TOTAL		=> 525,
		V_SYNC		=> 2,
		V_BACKP		=> 33,
		V_ACTIVE	=> 480
	)
	port map (
		video_clk	=> videoclk_sig,
		reset		=> not reset_n_sig,
		scan_ena	=> '1',
		dither_ena	=> '0',
		test_hcount	=> hcount_sig,
		test_vcount => vcount_sig,
		framestart	=> open,
		linestart	=> open,
		dither		=> open,
		pixelena	=> open,
		hsync		=> hsync_sig,
		vsync		=> vsync_sig,
		hblank		=> hblank_sig,
		vblank		=> vblank_sig
	);
end generate;

SVGA : if (RESOLUTION = "SVGA") generate
	pll_inst : pll_video_svga PORT MAP (
		areset	 => not reset_n,
		inclk0	 => clk_50mhz,
		c0	 	 => videoclk_sig,	-- 40.0MHz
		locked	 => reset_n_sig
	);

	vgen_inst : vga_syncgen
	generic map (
		H_TOTAL		=> 1056,	-- SVGA
		H_SYNC		=> 128,
		H_BACKP		=> 88,
		H_ACTIVE	=> 800,
		V_TOTAL		=> 628,
		V_SYNC		=> 4,
		V_BACKP		=> 23,
		V_ACTIVE	=> 600
	)
	port map (
		video_clk	=> videoclk_sig,
		reset		=> not reset_n_sig,
		scan_ena	=> '1',
		dither_ena	=> '0',
		test_hcount	=> hcount_sig,
		test_vcount => vcount_sig,
		framestart	=> open,
		linestart	=> open,
		dither		=> open,
		pixelena	=> open,
		hsync		=> hsync_sig,
		vsync		=> vsync_sig,
		hblank		=> hblank_sig,
		vblank		=> vblank_sig
	);
end generate;

XGA : if (RESOLUTION = "XGA") generate
	pll_inst : pll_video_xga PORT MAP (
		areset	 => not reset_n,
		inclk0	 => clk_50mhz,
		c0	 	 => videoclk_sig,	-- 65.0MHz
		locked	 => reset_n_sig
	);

	vgen_inst : vga_syncgen
	generic map (
		H_TOTAL		=> 1344,	-- XGA
		H_SYNC		=> 17,
		H_BACKP		=> 20,
		H_ACTIVE	=> 1024,
		V_TOTAL		=> 806,
		V_SYNC		=> 6,
		V_BACKP		=> 29,
		V_ACTIVE	=> 768
	)
	port map (
		video_clk	=> videoclk_sig,
		reset		=> not reset_n_sig,
		scan_ena	=> '1',
		dither_ena	=> '0',
		test_hcount	=> hcount_sig,
		test_vcount => vcount_sig,
		framestart	=> open,
		linestart	=> open,
		dither		=> open,
		pixelena	=> open,
		hsync		=> hsync_sig,
		vsync		=> vsync_sig,
		hblank		=> hblank_sig,
		vblank		=> vblank_sig
	);
end generate;

SD480P : if (RESOLUTION = "480p") generate
	pll_inst : pll_video_sd PORT MAP (
		areset	 => not reset_n,
		inclk0	 => clk_50mhz,
		c0	 	 => videoclk_sig,	-- 27.0MHz
		locked	 => reset_n_sig
	);

	vgen_inst : vga_syncgen
	generic map (
		H_TOTAL		=> 858,	-- 480p
		H_SYNC		=> 63,
		H_BACKP		=> 59,
		H_ACTIVE	=> 720,
		V_TOTAL		=> 525,
		V_SYNC		=> 6,
		V_BACKP		=> 30,
		V_ACTIVE	=> 483
	)
	port map (
		video_clk	=> videoclk_sig,
		reset		=> not reset_n_sig,
		scan_ena	=> '1',
		dither_ena	=> '0',
		test_hcount	=> hcount_sig,
		test_vcount => vcount_sig,
		framestart	=> open,
		linestart	=> open,
		dither		=> open,
		pixelena	=> open,
		hsync		=> hsync_sig,
		vsync		=> vsync_sig,
		hblank		=> hblank_sig,
		vblank		=> vblank_sig
	);
end generate;

HD720P : if (RESOLUTION = "720p") generate
	pll_inst : pll_video_hd PORT MAP (
		areset	 => not reset_n,
		inclk0	 => clk_50mhz,
		c0	 	 => videoclk_sig,	-- 74.25MHz
		locked	 => reset_n_sig
	);

	vgen_inst : vga_syncgen
	generic map (
		H_TOTAL		=> 1650,	-- 720p
		H_SYNC		=> 40,
		H_BACKP		=> 260,
		H_ACTIVE	=> 1280,
		V_TOTAL		=> 750,
		V_SYNC		=> 5,
		V_BACKP		=> 20,
		V_ACTIVE	=> 720
	)
	port map (
		video_clk	=> videoclk_sig,
		reset		=> not reset_n_sig,
		scan_ena	=> '1',
		dither_ena	=> '0',
		test_hcount	=> hcount_sig,
		test_vcount => vcount_sig,
		framestart	=> open,
		linestart	=> open,
		dither		=> open,
		pixelena	=> open,
		hsync		=> hsync_sig,
		vsync		=> vsync_sig,
		hblank		=> hblank_sig,
		vblank		=> vblank_sig
	);
end generate;

HD1080P30 : if (RESOLUTION = "1080p/30") generate
	pll_inst : pll_video_hd PORT MAP (
		areset	 => not reset_n,
		inclk0	 => clk_50mhz,
		c0	 	 => videoclk_sig,	-- 74.25MHz
		locked	 => reset_n_sig
	);

	vgen_inst : vga_syncgen
	generic map (
		H_TOTAL		=> 2200,	-- 1080p/30
		H_SYNC		=> 44,
		H_BACKP		=> 192,
		H_ACTIVE	=> 1920,
		V_TOTAL		=> 1125,
		V_SYNC		=> 5,
		V_BACKP		=> 36,
		V_ACTIVE	=> 1080
	)
	port map (
		video_clk	=> videoclk_sig,
		reset		=> not reset_n_sig,
		scan_ena	=> '1',
		dither_ena	=> '0',
		test_hcount	=> hcount_sig,
		test_vcount => vcount_sig,
		framestart	=> open,
		linestart	=> open,
		dither		=> open,
		pixelena	=> open,
		hsync		=> hsync_sig,
		vsync		=> vsync_sig,
		hblank		=> hblank_sig,
		vblank		=> vblank_sig
	);
end generate;


	de_sig <= (not hblank_sig) and (not vblank_sig);

	pix_r_sig <= hcount_sig(7 downto 0);
	pix_g_sig <= vcount_sig(3 downto 0) & hcount_sig(9 downto 6);
	pix_b_sig <= vcount_sig(9 downto 2);


	U1 : dvi_tx_pdiff
	generic map (
		RESOLUTION	=> RESOLUTION
	)
	port map (
		reset		=> not reset_n_sig,
		clk			=> videoclk_sig,
		test_locked	=> locked_sig,

		dvi_de		=> de_sig,
		dvi_blu		=> pix_b_sig,
		dvi_grn		=> pix_g_sig,
		dvi_red		=> pix_r_sig,
		dvi_hsync	=> hsync_sig,
		dvi_vsync	=> vsync_sig,
		dvi_ctl		=> "0000",

		data0_p		=> dvi_data0_p,
		data0_n		=> dvi_data0_n,
		data1_p		=> dvi_data1_p,
		data1_n		=> dvi_data1_n,
		data2_p		=> dvi_data2_p,
		data2_n		=> dvi_data2_n,
		clock_p		=> dvi_clock_p,
		clock_n		=> dvi_clock_n
	);


end RTL;



----------------------------------------------------------------------
--   Copyright (C)2010-2013 J-7SYSTEM Works.  All rights Reserved.  --
----------------------------------------------------------------------
